"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var react_1 = require('react');
var isObject = require('lodash.isobject');
var isEqual = require('lodash.isequal');
var invariant = require('invariant');
var assign = require('object-assign');
var react_redux_1 = require('react-redux');
var apollo_client_1 = require('apollo-client');
;
;
;
var defaultMapQueriesToProps = function (opts) { return ({}); };
var defaultMapMutationsToProps = function (opts) { return ({}); };
var defaultQueryData = {
    loading: true,
    errors: null,
};
var defaultMutationData = assign({}, defaultQueryData);
function getDisplayName(WrappedComponent) {
    return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}
var nextVersion = 0;
function connect(opts) {
    if (!opts) {
        opts = {};
    }
    var mapQueriesToProps = opts.mapQueriesToProps, mapMutationsToProps = opts.mapMutationsToProps;
    delete opts.mapQueriesToProps;
    delete opts.mapMutationsToProps;
    mapQueriesToProps = mapQueriesToProps ? mapQueriesToProps : defaultMapQueriesToProps;
    mapMutationsToProps = mapMutationsToProps ? mapMutationsToProps : defaultMapMutationsToProps;
    var version = nextVersion++;
    return function wrapWithApolloComponent(WrappedComponent) {
        var apolloConnectDisplayName = "Apollo(Connect(" + getDisplayName(WrappedComponent) + "))";
        var ApolloConnect = (function (_super) {
            __extends(ApolloConnect, _super);
            function ApolloConnect(props, context) {
                _super.call(this, props, context);
                this.version = version;
                this.store = props.store || context.store;
                this.client = props.client || context.client;
                invariant(!!this.client, "Could not find \"client\" in either the context or " +
                    ("props of \"" + apolloConnectDisplayName + "\". ") +
                    "Either wrap the root component in a <Provider>, " +
                    ("or explicitly pass \"client\" as a prop to \"" + apolloConnectDisplayName + "\"."));
                var storeState = this.store.getState();
                this.state = assign({}, storeState);
                this.data = {};
                this.mutations = {};
            }
            ApolloConnect.prototype.componentWillMount = function () {
                var _a = this, props = _a.props, state = _a.state;
                this.subscribeToAllQueries(props, state);
                this.createAllMutationHandles(props, state);
            };
            ApolloConnect.prototype.componentWillReceiveProps = function (nextProps) {
                if (!isEqual(this.props, nextProps)) {
                    this.haveOwnPropsChanged = true;
                    this.unsubcribeAllQueries();
                    this.subscribeToAllQueries(nextProps, this.state);
                }
            };
            ApolloConnect.prototype.shouldComponentUpdate = function (nextProps, nextState) {
                return this.haveOwnPropsChanged ||
                    this.hasQueryDataChanged ||
                    this.hasMutationDataChanged;
            };
            ApolloConnect.prototype.componentWillUnmount = function () {
                this.unsubcribeAllQueries();
            };
            ApolloConnect.prototype.subscribeToAllQueries = function (props, state) {
                var _a = this.client, watchQuery = _a.watchQuery, reduxRootKey = _a.reduxRootKey;
                var store = this.store;
                var queryHandles = mapQueriesToProps({
                    state: store.getState(),
                    ownProps: props,
                });
                if (isObject(queryHandles) && Object.keys(queryHandles).length) {
                    this.queryHandles = queryHandles;
                    for (var key in queryHandles) {
                        if (!queryHandles.hasOwnProperty(key)) {
                            continue;
                        }
                        var _b = queryHandles[key], query = _b.query, variables = _b.variables;
                        var handle = watchQuery(queryHandles[key]);
                        var queryData = defaultQueryData;
                        try {
                            var result = apollo_client_1.readQueryFromStore({
                                store: store.getState()[reduxRootKey].data,
                                query: query,
                                variables: variables,
                            });
                            queryData = assign({
                                errors: null,
                                loading: false,
                            }, result);
                        }
                        catch (e) { }
                        this.data[key] = queryData;
                        this.handleQueryData(handle, key);
                    }
                }
            };
            ApolloConnect.prototype.unsubcribeAllQueries = function () {
                if (this.queryHandles) {
                    for (var key in this.queryHandles) {
                        if (!this.queryHandles.hasOwnProperty(key)) {
                            continue;
                        }
                        this.queryHandles[key].unsubscribe();
                    }
                }
            };
            ApolloConnect.prototype.handleQueryData = function (handle, key) {
                var _this = this;
                var refetch, startPolling, stopPolling;
                var createBoundRefetch = function (dataKey, refetchMethod) {
                    return function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i - 0] = arguments[_i];
                        }
                        _this.data[dataKey] = assign(_this.data[dataKey], {
                            loading: true,
                            refetch: refetch,
                        });
                        _this.hasQueryDataChanged = true;
                        _this.setState(_this.store.getState());
                        refetchMethod.apply(void 0, args);
                    };
                };
                var oldData = {};
                var forceRender = function (_a) {
                    var errors = _a.errors, _b = _a.data, data = _b === void 0 ? {} : _b;
                    var resultKeyConflict = ('errors' in data ||
                        'loading' in data ||
                        'refetch' in data ||
                        'startPolling' in data ||
                        'stopPolling' in data);
                    invariant(!resultKeyConflict, ("the result of the '" + key + "' query contains keys that ") +
                        "conflict with the return object. 'errors', 'loading', " +
                        "'startPolling', 'stopPolling', and 'refetch' cannot be " +
                        "returned keys");
                    if (!isEqual(oldData, data) || errors) {
                        _this.hasQueryDataChanged = true;
                    }
                    oldData = assign({}, data);
                    _this.data[key] = assign({
                        loading: false,
                        errors: errors,
                        refetch: refetch,
                        startPolling: startPolling,
                        stopPolling: stopPolling,
                    }, data);
                    _this.setState(_this.store.getState());
                };
                this.queryHandles[key] = handle.subscribe({
                    next: forceRender,
                    error: function (errors) { forceRender({ errors: errors }); },
                });
                refetch = createBoundRefetch(key, this.queryHandles[key].refetch);
                startPolling = this.queryHandles[key].startPolling;
                stopPolling = this.queryHandles[key].stopPolling;
                this.data[key] = assign(this.data[key], {
                    refetch: refetch,
                    startPolling: startPolling,
                    stopPolling: stopPolling,
                });
            };
            ApolloConnect.prototype.createAllMutationHandles = function (props, state) {
                var mutations = mapMutationsToProps({
                    state: state,
                    ownProps: props,
                });
                if (isObject(mutations) && Object.keys(mutations).length) {
                    for (var key in mutations) {
                        if (!mutations.hasOwnProperty(key)) {
                            continue;
                        }
                        var handle = this.createMutationHandle(key, mutations[key]);
                        this.data[key] = defaultMutationData;
                        this.mutations[key] = handle;
                    }
                }
            };
            ApolloConnect.prototype.createMutationHandle = function (key, method) {
                var _this = this;
                var mutate = this.client.mutate;
                var store = this.store;
                var forceRender = function (_a) {
                    var errors = _a.errors, _b = _a.data, data = _b === void 0 ? {} : _b;
                    var resultKeyConflict = ('errors' in data ||
                        'loading' in data ||
                        'refetch' in data);
                    invariant(!resultKeyConflict, ("the result of the '" + key + "' mutation contains keys that ") +
                        "conflict with the return object. 'errors', 'loading', and 'refetch' cannot be " +
                        "returned keys");
                    _this.data[key] = assign({
                        loading: false,
                        errors: errors,
                    }, data);
                    _this.hasMutationDataChanged = true;
                    _this.setState(store.getState());
                    return {
                        errors: errors,
                        data: data,
                    };
                };
                return function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i - 0] = arguments[_i];
                    }
                    var _a = method.apply(_this.client, args), mutation = _a.mutation, variables = _a.variables;
                    return new Promise(function (resolve, reject) {
                        _this.data[key] = assign(_this.data[key], {
                            loading: true,
                        });
                        _this.hasMutationDataChanged = true;
                        _this.setState(store.getState());
                        resolve();
                    })
                        .then(function () {
                        return mutate({ mutation: mutation, variables: variables });
                    })
                        .then(forceRender)
                        .catch(function (errors) { return forceRender({ errors: errors }); });
                };
            };
            ApolloConnect.prototype.render = function () {
                var _a = this, haveOwnPropsChanged = _a.haveOwnPropsChanged, hasQueryDataChanged = _a.hasQueryDataChanged, hasMutationDataChanged = _a.hasMutationDataChanged, renderedElement = _a.renderedElement, mutations = _a.mutations, props = _a.props, data = _a.data;
                this.haveOwnPropsChanged = false;
                this.hasQueryDataChanged = false;
                this.hasMutationDataChanged = false;
                var clientProps = {
                    mutate: this.client.mutate,
                    query: this.client.query,
                };
                if (Object.keys(mutations).length) {
                    clientProps.mutations = mutations;
                }
                var mergedPropsAndData = assign({}, props, data, clientProps);
                if (!haveOwnPropsChanged &&
                    !hasQueryDataChanged &&
                    !hasMutationDataChanged &&
                    renderedElement) {
                    return renderedElement;
                }
                this.renderedElement = react_1.createElement(WrappedComponent, mergedPropsAndData);
                return this.renderedElement;
            };
            ApolloConnect.displayName = apolloConnectDisplayName;
            ApolloConnect.WrappedComponent = WrappedComponent;
            ApolloConnect.contextTypes = {
                store: react_1.PropTypes.object.isRequired,
                client: react_1.PropTypes.object.isRequired,
            };
            return ApolloConnect;
        }(react_1.Component));
        var mapStateToProps = opts.mapStateToProps, mapDispatchToProps = opts.mapDispatchToProps, mergeProps = opts.mergeProps, options = opts.options;
        return react_redux_1.connect(mapStateToProps, mapDispatchToProps, mergeProps, options)(ApolloConnect);
    };
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = connect;
;
//# sourceMappingURL=connect.js.map